@@ -34,8 +34,6 @@
 #include "t-support.h"
 
 
-static int got_errors;
-
 static const char test_text1[] = "Hallo Leute!\n";
 static const char test_text1f[]= "Hallo Leute?\n";
 static const char test_sig1[] =
@@ -62,78 +60,54 @@ check_result (gpgme_verify_result_t result, int summary, char *fpr,
     {
       fprintf (stderr, "%s:%i: Unexpected number of signatures\n",
 	       __FILE__, __LINE__);
-      got_errors = 1;
-      if (!sig)
-        return;
+      exit (1);
     }
   if (sig->summary != summary)
     {
       fprintf (stderr, "%s:%i: Unexpected signature summary: "
                "want=0x%x have=0x%x\n",
 	       __FILE__, __LINE__, summary, sig->summary);
-      got_errors = 1;
+      exit (1);
     }
-  if (sig->fpr && strcmp (sig->fpr, fpr))
+  if (strcmp (sig->fpr, fpr))
     {
       fprintf (stderr, "%s:%i: Unexpected fingerprint: %s\n",
 	       __FILE__, __LINE__, sig->fpr);
-      got_errors = 1;
+      exit (1);
     }
-  if (gpgme_err_code (sig->status) != status)
+  if (gpg_err_code (sig->status) != status)
     {
       fprintf (stderr, "%s:%i: Unexpected signature status: %s\n",
 	       __FILE__, __LINE__, gpgme_strerror (sig->status));
-      got_errors = 1;
+      exit (1);
     }
   if (sig->notations)
     {
       fprintf (stderr, "%s:%i: Unexpected notation data\n",
 	       __FILE__, __LINE__);
-      got_errors = 1;
+      exit (1);
     }
   if (sig->wrong_key_usage)
     {
       fprintf (stderr, "%s:%i: Unexpectedly wrong key usage\n",
 	       __FILE__, __LINE__);
-      got_errors = 1;
+      exit (1);
     }
   if (sig->validity != validity)
     {
       fprintf (stderr, "%s:%i: Unexpected validity: %i\n",
 	       __FILE__, __LINE__, sig->validity);
-      got_errors = 1;
+      exit (1);
     }
-  if (gpgme_err_code (sig->validity_reason) != GPG_ERR_NO_ERROR)
+  if (gpg_err_code (sig->validity_reason) != GPG_ERR_NO_ERROR)
     {
       fprintf (stderr, "%s:%i: Unexpected validity reason: %s\n",
 	       __FILE__, __LINE__, gpgme_strerror (sig->validity_reason));
-      got_errors = 1;
+      exit (1);
     }
 }
 
 
-static void
-show_auditlog (gpgme_ctx_t ctx)
-{
-  gpgme_error_t err;
-  gpgme_data_t data;
-
-  err = gpgme_data_new (&data);
-  fail_if_err (err);
-  err = gpgme_op_getauditlog (ctx, data, 0);
-  if (err)
-    {
-      fprintf (stderr, "%s:%i: Can't get audit log: %s\n",
-	       __FILE__, __LINE__, gpgme_strerror (err));
-      if (gpgme_err_code (err) != GPG_ERR_ASS_UNKNOWN_CMD)
-	got_errors = 1;
-    }
-  print_data (data);
-  gpgme_data_release (data);
-}
-
-
-
 int 
 main (int argc, char **argv)
 {
@@ -161,8 +135,6 @@ main (int argc, char **argv)
 		"3CF405464F66ED4A7DF45BBDD1E4282E33BDB76E",
 		GPG_ERR_NO_ERROR, GPGME_VALIDITY_FULL);
 
-  show_auditlog (ctx);
-
   /* Checking a manipulated message.  */
   gpgme_data_release (text);
   err = gpgme_data_new_from_mem (&text, test_text1f, strlen (test_text1f), 0);
@@ -175,10 +147,6 @@ main (int argc, char **argv)
 		"3CF405464F66ED4A7DF45BBDD1E4282E33BDB76E",
 		GPG_ERR_BAD_SIGNATURE, GPGME_VALIDITY_UNKNOWN);
 
-  show_auditlog (ctx);
-
-  gpgme_data_release (text);
-  gpgme_data_release (sig);
   gpgme_release (ctx);  
-  return got_errors? 1 : 0;
+  return 0;
 }
